include( CMakeParseArguments )

find_program( PROG_RPM rpm )
find_program( PROG_DNF dnf )
find_program( PROG_YUM yum )

function( echo )
  execute_process( COMMAND ${CMAKE_COMMAND} -E echo ${ARGN})
endfunction( )

function( DebBuildDependencies VAR )
  file( GLOB_RECURSE files "${TOP_DIR}/control" "${TOP_DIR}/control.in" )
  set( keepers )
  foreach( file ${files})
    if ( NOT file MATCHES /legacy/ )
      list( APPEND keepers ${file} )
    endif( )
  endforeach( )
  set( files ${keepers} )
  # message( STATUS "files: ${files}" )
  unset(deps)
  foreach( SPEC_FILE ${files} )
    file( READ ${SPEC_FILE} dependencies )
    string( CONFIGURE "${dependencies}" dependencies @ONLY )
    # Remove any undefined substitution variables
    string( REGEX REPLACE '@[A-Za-z0-9_]*@' "_junk_" dependencies "${dependencies}" )
    string( REGEX REPLACE "\n" ";" dependencies "${dependencies}" )
    set( stage 1 )
    foreach(line ${dependencies})
      string(REGEX REPLACE "[ \t\n]+$" "" line "${line}" )
      string(REGEX REPLACE "^[ \t\n]+" "" line "${line}" )
      if ( stage EQUAL 1 )
        if( "${line}" MATCHES "^[Bb]uild-[Dd]epends:" )
          string( REGEX REPLACE "^[Bb]uild-[Dd]epends:" "" line "${line}" )
          string( REGEX REPLACE "[,|]" ";" line "${line}" )
          string( REGEX REPLACE "[ \t]+" "" line "${line}" )
          list( APPEND deps ${line} )
          set( stage 2 )
        endif ( )
      elseif( stage EQUAL 2 )
        if ( NOT "${line}" MATCHES ",[ \t]*$" )
          set( stage 1 )
        endif( )
        string( REGEX REPLACE "[,|]" ";" line "${line}" )
        string( REGEX REPLACE "[ \t]+" "" line "${line}" )
        list( APPEND deps ${line} )
      endif( )
    endforeach( )
    string( REGEX REPLACE "[(][^)]*[)]" "" deps "${deps}")
    #list( REMOVE_ITEM deps debhelper )
    # message( STATUS "DEB dependencies: ${deps}")
  endforeach( )
  list( SORT deps )
  list( REMOVE_DUPLICATES deps )
  set( ${VAR} ${deps} PARENT_SCOPE )
endfunction( )

function( RPMInstallPackage PACKAGE )
  set(options
    )
  set(oneValueArgs
    )
  set(multiValueArgs
    )

  cmake_parse_arguments(ARG "${options}" "${oneValueArgs}" "${multiValueArgs}" ${ARGN} )

  if ( PROG_DNF )
    execute_process(
      COMMAND ${PROG_DNF} install --assumeyes ${PACKAGE}
      OUTPUT_VARIABLE local_output
      ERROR_VARIABLE local_error
      )
  elseif( PROG_YUM )
    execute_process(
      COMMAND ${PROG_YUM} install --assumeyes ${PACKAGE}
      OUTPUT_VARIABLE local_output
      ERROR_VARIABLE local_error
      )
  endif( )
  unset( local_output )
  unset( local_error )

endfunction( )

##
# @fn RPMListDependencies( SPEC_FILE VAR )
#
# @brief List build dependencies from given SPEC_FILE
#
# @details
# This function will list BuildRequires dependencies
# specified in the SPEC_FILE.
#
# @param[in] SPEC_FILE
#   RPM spec file; must have the extension .spec
# @param[out] VAR
#   name of variable to substitute with result
#
# @see @ref RPMInstallDependencies
#
function( RPMListDependencies SPEC_FILE VAR )
  RPMInstallPackage( rpm-build )
  find_program( PROG_RPMSPEC rpmspec )

  message( INFO "PROG_RPMSPEC: ${PROG_RPMSPEC}")
  message( INFO "PROG_RPM: ${PROG_RPM}")
  set( dependencies "" )
  if ( PROG_RPMSPEC )
    execute_process(
      COMMAND ${PROG_RPMSPEC} --buildrequires --query "${SPEC_FILE}"
      OUTPUT_VARIABLE dependencies
      ERROR_VARIABLE dependencies_error
      )
  elseif ( PROG_RPM )
    file( READ ${SPEC_FILE} dependencies )
    execute_process(
      COMMAND ${PROG_RPM} --eval "${dependencies}"
      OUTPUT_VARIABLE raw_dependencies
      ERROR_VARIABLE dependencies_error
      )
    set( deps "" )
    foreach(line ${raw_dependencies})
      if( "${line}" MATCHES "^[Bb]uild[Rr]equires:" )
        string(REGEX REPLACE "^[Bb]uild[Rr]equires:[ \t]*" "" line ${line})
        list(APPEND dependencies ${line})
      endif( )
    endforeach( )
  endif( )
  message(WARNING "dependencies ${dependencies}")
  message(WARNING "dependencies_error ${dependencies_error}")
  string( REGEX REPLACE "\n" ";" dependencies "${dependencies}")
  foreach(line ${dependencies})
    string(REGEX REPLACE ">*=[ \t]*((([0-9]+)[.])*[0-9]+)*[ \t]*," "," line ${line})
    string(REGEX REPLACE ">*=[ \t]*[@][^@]*[@][ \t]*," "," line ${line})
    string(REGEX REPLACE ">*=[ \t]*((([0-9]+)[.])*[0-9]+)*$" "" line ${line})
    string(REGEX REPLACE ">*=[ \t]*[@][^@]*[@]$" "" line ${line})
    string(REGEX REPLACE "," ";" line ${line})
    foreach(pkg_ ${line})
      string(STRIP "${pkg_}" pkg_)
      list(APPEND deps ${pkg_})
    endforeach()
  endforeach( )
  set( ${VAR} ${deps} PARENT_SCOPE )
endfunction( )

##
## @fn RPMInstallDependencies( SPEC_FILE VAR )
##
## @brief Installs build dependencies from given SPEC_FILE
##
## @details
## This function will attempt to install BuildRequires dependencies
## specified in the SPEC_FILE.
##
## @param[in] SPEC_FILE
##   RPM spec file; must have the extension .spec
## @param[out] VAR
##   name of variable to substitute with result
##
## @note
##   If there is no known way to install the dependencies,
##   a list of packages to be installed will be returned
##
## @see @ref RPMListDependencies
##
function( RPMInstallDependencies SPEC_FILE VAR )
  if( INSTALL )
    message( INFO "INFO: DEP: PROG_DNF: ${PROG_DNF}")
    message( INFO "INFO: DEP: PROG_YUM: ${PROG_YUM}")
    execute_process(
      COMMAND rpmspec --buildrequires -q "${SPEC_FILE}"
      OUTPUT_VARIABLE dependencies
      ERROR_VARIABLE dependencies_error
      )
    message(WARNING "INFO: DEP: Dependencies to install: ${dependencies}" )
    if ( PROG_DNF )
      execute_process(
        COMMAND dnf builddep -y "${SPEC_FILE}"
        OUTPUT_VARIABLE dependencies
        ERROR_VARIABLE dependencies_error
        )
    elseif( PROG_YUM )
      RPMInstallPackage( yum-utils )
      find_program( PROG_YUM_BUILDDEP yum-builddep )
      if ( PROG_YUM_BUILDDEP )
        message( INFO " COMMAND: ${PROG_YUM_BUILDDEP} --assumeyes --verbose --debuglevel=10 \"${SPEC_FILE}\"" )
        execute_process(
          COMMAND ${PROG_YUM_BUILDDEP} --assumeyes --verbose --debuglevel=10 "${SPEC_FILE}"
          OUTPUT_VARIABLE dependencies
          ERROR_VARIABLE dependencies_error
          )
      endif( )
    else( )
      RPMListDependencies( ${SPEC_FILE} dependencies )
    endif( )
    set( ${VAR} ${dependencies} PARENT_SCOPE )
  endif( )
endfunction( )

function( RPMBuildDependencies VAR )
  file( GLOB_RECURSE files "${TOP_DIR}/*.spec" "${TOP_DIR}/*.spec.in" )
  set( keepers )
  foreach( file ${files})
    if ( NOT file MATCHES /legacy/
        AND NOT file MATCHES "[/][.][^/]*$" )
      list( APPEND keepers ${file} )
    endif( )
  endforeach( )
  set( files ${keepers} )
  message( WARNING "files: ${files}" )
  unset(deps)
  foreach( SPEC_FILE ${files} )
    file( READ ${SPEC_FILE} dependencies )
    string( CONFIGURE "${dependencies}" dependencies @ONLY )
    # Remove any undefined substitution variables
    string( REGEX REPLACE '@[A-Za-z0-9_]*@' "_junk_" dependencies "${dependencies}" )
    message(WARNING "configured dependencies ${dependencies}")
    get_filename_component(SPEC_FILE_EXPANDED "${SPEC_FILE}" NAME)
    set( SPEC_FILE_EXPANDED "/tmp/${SPEC_FILE_EXPANDED}" )
    string(REGEX REPLACE "[.]in$" "" SPEC_FILE_EXPANDED "${SPEC_FILE_EXPANDED}" )
    file( WRITE ${SPEC_FILE_EXPANDED} "${dependencies}" )
    file( READ ${SPEC_FILE_EXPANDED} dependencies )
    message(WARNING "INFO: DEP: Contents of ${SPEC_FILE_EXPANDED}: ${dependencies}")
    message(WARNING "INFO: DEP: INSTALL: ${INSTALL}")
    if ( INSTALL )
      RPMInstallDependencies( ${SPEC_FILE_EXPANDED} deps )
    else( )
      RPMListDependencies( ${SPEC_FILE_EXPANDED} deps )
    endif( )
    file( REMOVE ${SPEC_FILE_EXPANDED} )
  endforeach( )
  if ( deps )
    list( SORT deps )
    list( REMOVE_DUPLICATES deps )
  endif( )
  message( WARNING "deps ${deps}" )
  set( ${VAR} ${deps} PARENT_SCOPE )
endfunction( )

set( VERBOSE True )
if( NOT PYTHON3_VERSION_NODOTS )
  if ( PYTHON3_VERSION )
    set( PYTHON3_VERSION_NODOTS ${PYTHON3_VERSION} )
    string( REPLACE "." "" PYTHON3_VERSION_NODOTS "${PYTHON3_VERSION_NODOTS}" )
    set( PYTHON3_VERSION_NODOTS "${PYTHON3_VERSION_NODOTS}" CACHE INTERNAL "" )
  endif( )
endif( )
if( NOT PYTHON2_VERSION_NODOTS )
  if ( PYTHON2_VERSION )
    set( PYTHON2_VERSION_NODOTS ${PYTHON2_VERSION} )
    string( REPLACE "." "" PYTHON2_VERSION_NODOTS "${PYTHON2_VERSION_NODOTS}" )
    set( PYTHON2_VERSION_NODOTS ${PYTHON2_VERSION_NODOTS} CACHE INTERNAL "" )
  endif( )
endif( )
if ( PACKAGER STREQUAL "rpm" )
  RPMBuildDependencies( depends )
elseif ( PACKAGER STREQUAL "deb" )
  DebBuildDependencies( depends )
else( )
  set( VERBOSE False )
endif( )
if ( VERBOSE )
  if ( DEFINED INCLUDE_PATTERN )
    if ( NOT "${INCLUDE_PATTERN}" MATCHES "^[\\^]")
      set(INCLUDE_PATTERN "^.*${INCLUDE_PATTERN}")
    endif( )
    if ( NOT "${INCLUDE_PATTERN}" MATCHES "[\\$]$")
      set(INCLUDE_PATTERN "${INCLUDE_PATTERN}.*$")
    endif( )
    foreach( dep ${depends} )
      if ( "${dep}" MATCHES "${INCLUDE_PATTERN}" )
        list(APPEND new_depends "${dep}" )
      endif( )
    endforeach( )
    set(depends ${new_depends})
  endif( )
  if ( NOT DEFINED INSTALL )
    set(INSTALL FALSE )
  endif( )
  if ( DEFINED EXCLUDE_PATTERN )
    if ( NOT "${EXCLUDE_PATTERN}" MATCHES "^[\\^]")
      set(EXCLUDE_PATTERN "^.*${EXCLUDE_PATTERN}")
    endif( )
    if ( NOT "${EXCLUDE_PATTERN}" MATCHES "[\\$]$")
      set(EXCLUDE_PATTERN "${EXCLUDE_PATTERN}.*$")
    endif( )
    foreach( dep ${depends} )
      if ( NOT "${dep}" MATCHES "${EXCLUDE_PATTERN}" )
        list(APPEND new_depends "${dep}" )
      endif( )
    endforeach( )
    set(depends ${new_depends})
  endif( )
  foreach( dep ${depends} )
    echo( ${dep} )
  endforeach( )
endif( )
